<?php

declare(strict_types=1);

namespace Drupal\graphql_compose_preview\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\Attribute;
use Drupal\graphql_compose_preview\TokenHelper;
use Drupal\token\TokenInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'preview_token_iframe' formatter.
 *
 * @FieldFormatter(
 *   id = "preview_token_iframe",
 *   label = @Translation("Token preview iframe"),
 *   field_types = {
 *     "preview_token",
 *   },
 * )
 */
class PreviewTokenIframeFormatter extends FormatterBase {

  /**
   * The token helper service.
   *
   * @var \Drupal\graphql_compose_preview\TokenHelper
   */
  protected TokenHelper $tokenHelper;

  /**
   * The token service.
   *
   * @var \Drupal\token\TokenInterface
   */
  protected TokenInterface $token;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->tokenHelper = $container->get('graphql_compose_preview.token_helper');
    $instance->token = $container->get('token');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'iframe_url' => 'https://my.frontend/preview/[node:preview_uuid]/[node:preview_token]',
      'class' => '',
      'width' => '100%',
      'height' => '500',
      'allow' => 'fullscreen autoplay',
      'transparency' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Displays as iframe.');
    $summary[] = $this->t('URL: @iframe_url', ['@iframe_url' => $this->getSetting('iframe_url')]);

    if ($this->getSetting('class')) {
      $summary[] = $this->t('CSS class: @class', ['@class' => $this->getSetting('class')]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      'iframe_url' => [
        '#type' => 'textfield',
        '#title' => $this->t('Base URL'),
        '#default_value' => $this->getSetting('iframe_url'),
        '#description' => $this->t('The URL for your iframe. Use the tokens [node:preview_uuid] and [node:preview_token].'),
        '#token_types' => ['node'],
      ],
      'token_help' => [
        '#theme' => 'token_tree_link',
        '#token_types' => ['node'],
      ],
      'class' => [
        '#type' => 'textfield',
        '#title' => $this->t('CSS class'),
        '#default_value' => $this->getSetting('class'),
        '#description' => $this->t('The CSS class to apply to the iframe.'),
      ],
      'width' => [
        '#type' => 'textfield',
        '#title' => $this->t('Width'),
        '#default_value' => $this->getSetting('width'),
        '#description' => $this->t('The width of the iframe.'),
      ],
      'height' => [
        '#type' => 'textfield',
        '#title' => $this->t('Height'),
        '#default_value' => $this->getSetting('height'),
        '#description' => $this->t('The height of the iframe.'),
      ],
      'allow' => [
        '#type' => 'textfield',
        '#title' => $this->t('Allow'),
        '#default_value' => $this->getSetting('allow'),
        '#description' => $this->t('The allow attribute of the iframe.'),
      ],
      'transparency' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Allow transparency'),
        '#default_value' => $this->getSetting('transparency'),
        '#description' => $this->t('Allow transparency in the iframe.'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = [];

    $entity = $items->getEntity();
    $url = $this->tokenHelper->url($entity);
    $access = $this->tokenHelper->access($entity);

    $src = $this->token->replace(
      $this->getSetting('iframe_url'),
      ['node' => $entity],
      ['clear' => FALSE]
    );

    foreach ($items as $delta => $item) {
      $element[$delta] = [
        '#theme' => 'token_preview_iframe',
        '#node' => $entity,
        '#preview_token' => $item,
        '#preview_token_url' => $url,
        '#preview_token_access' => $access,
        '#attributes' => new Attribute([
          'src' => $src,
          'class' => $this->getSetting('class'),
          'width' => $this->getSetting('width'),
          'height' => $this->getSetting('height'),
          'allow' => $this->getSetting('allow'),
          'allowtransparency' => $this->getSetting('transparency'),
          'frameborder' => 0,
        ]),
      ];
    }

    return $element;
  }

}
